// -------------------------------------------------------------------------- //
// Built-In Component Definitions
//
// Name                 Properties
// ----------           ----------
// Animation            frames, delay, duration, looped, global
// CameraTarget         enabled, top, bottom, left, right
// Collidable
// Emitter              emitter, x, y, layer, rate, attached
// Location             x, y, width, height
// Movement             vx, vy, vxMax, vyMax, gravity, frictionX, frictionY
// PlatformClipped
// PreciseClipped
// Script               script, timers
// Solid                solid, top, bottom, left, right, mass
// TileSensor
// Visible              lib, frame, layer, editorOnly
//
// KeenBehavior
// LittleKeenBehavior
//
// * = Static property (read-only, shared among all instances)
// -------------------------------------------------------------------------- //
[

// A component that plays a sound when its entity is first spawned.
//
// Variable     Type      Description
// --------     ----      -----------
// sound        string    The name of the sound to play.
//
{Component: "SpawnSound", Extends: "Script"
    Properties: {
        script: {
            member OnSpawned()
                PlaySound(this.sound);
            end function

            sound: null             // Required
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that attempts to activate other entities by sending an
// "ActivateOnTouch" message to all entities it comes into contact with. The
// other entity may react to the message in any way, or it may ignore it.
//
{Component: "Activator", Extends: "Script"
    Properties: {
        script: {
            member OnCollided(args : map)
                args.eid.Send("ActivateOnTouch", {eid: this.entity});
            end function
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that indicates its entity is climbable, by sending a "Climb"
// message in response to a "TryToClimb" message from the entity wishing to
// climb.
//
{Component: "Climbable", Extends: "Script"
    Properties: {
        script: {
            member OnTryToClimb(args : map)
                args.eid.Send("Climb", {eid: this.entity});
            end function
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A Visible component that is only visible in the level editor.
//
// Property    Type             Description
// --------    ----             -----------
// lib         string           Name of the sprite's tile library.
// frame       string/number    Frame name or tile id.
{Component: "EditorVisible", Extends: "Visible"
    Properties: {
        lib:        null            // Required
        frame:      null            // Required
        editorOnly: true
        layer:      3
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that gives a bonus when its activated by an entity on contact.
// After the item's bonus is collected, it self destructs.
//
// Variable    Type       Description
// --------    ----       -----------
// name        string     The name of the game value to receive a bonus.
// amount      number     Amount to add to the game value.
// particle    string     Name of the particle emitter to spawn.
// sound       string     Sound to play.
//
{Component: "Item", Extends: "Script"
    Properties: {
        script: {
            member OnActivateOnTouch()
                game[this.name] += this.amount;
                if type(this.sound) = "string" then
                    PlaySound(this.sound);
                end if;
                if type(this.particle) = "string" then
                    ParticleBurst(this.particle, this.entity.x, this.entity.y, 3, 1);
                end if;
                this.entity.Destroy();
            end function

            name:     null          // Required
            amount:   null          // Required
            particle: null
            sound:    null
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that is lethal on contact. It sends a Kill message on contact
// that specifies the type of entity it kills.
//
// Variable    Type       Description
// --------    ----       -----------
// victim      string     The type of entity to kill (required)
// enabled     boolean    Kill messages are enabled (default = true)
//
{Component: "Lethal", Extends: "Script"
    Properties: {
        script: {
            member OnCollided(args : map)
                if this.enabled then
                    args.eid.Send("Kill", {victim: this.victim});
                end if;
            end function

            member OnLethal(args : map)
                this.enabled := args.enabled;
            end function

            victim:  ""             // must be overridden
            enabled: true           // Kill messages will be sent
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that is lethal to Player entities on contact.
//
{Component: "LethalToPlayer", Extends: "Lethal"
    Properties: { script: { victim: "Player" } }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A Movement component parameterized for Keen's overhead maps.
//
{Component: "MapMovement", Extends: "Movement"
    Properties: {vxMax: 500, vyMax: 500, frictionX: 700, frictionY: 700}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A Movement component parameterized for Keen's platform levels.
//
{Component: "PlatformMovement", Extends: "Movement"
    Properties: {vxMax: 140, vyMax: 380, gravity: 768, frictionX: 800, frictionY: 0}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that destroys its own entity after a delay.
//
// Property    Type       Description
// --------    ----       -----------
// delay       number     Milliseconds to delay before destructing.
//
{Component: "SelfDestructing", Extends: "Script"
    Properties: {
        script: {
            member Expire()
                this.entity.Destroy();
            end function
        }
        timers: {Expire: 0}     // override the "Expire" timer period
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that indicates to a Shot that it has been blocked by the
// component's entity.
//
// Variable    Type       Description
// --------    ----       -----------
// shooter     string     Blocks shots produced by this shooter type.
//
{Component: "ShotBlocker", Extends: "Script"
    Properties: {
        script: {
            member OnHitByShot(args : map)
                if args.shooter = this.shooter then
                    args.eid.Send("ShotBlocked");
                end if;
            end function
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that blocks shots from enemies.
//
{Component: "EnemyShotBlocker", Extends: "ShotBlocker"
    Properties: {
        script: {
            shooter: "Enemy"
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that blocks shots from the Player entity.
//
{Component: "PlayerShotBlocker", Extends: "ShotBlocker"
    Properties: {
        script: {
            shooter: "Player"
        }
    }
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

// A component that spawns a new entity, on entity destruction, at its
// current location (with an optional location offset).
//
// Variable     Type     Description
// --------     ----     -----------
// template     string   The entity template to use.
// offsetX      number   X offset of new entity (default=0).
// offsetY      number   Y offset of new entity (default=0).
// properties   map      Properties for entity template (default={}).
//
{Component: "SpawnOnDestruct", Extends: "Script"
    Properties: {
        script: {
            member OnDestruct()
                SpawnAt(this.template,
                        this.entity.x + this.offsetX,
                        this.entity.y + this.offsetY,
                        this.properties);
            end function

            template: ""            // Required
            offsetX: 0
            offsetY: 0
            properties: {}
        }
    }
}

]
