////////////////////////////////////////////////////////////////////////////////
//                              ORACLE ENGINE API                             //
{

/////////////////////
// AUDIO FUNCTIONS //
/////////////////////

// PlayMusic(name : string) returns null
//
// Stops the music, if any is playing, and begins playing music 'filename'. If
// a file extension is not specified, it will be detected using the supported
// audio file formats.
function PlayMusic(filename : string) : PlayMusic

// StopMusic() returns null
//
// Stops the music, if any is playing.
function StopMusic() : StopMusic

// PlaySound(filename : string) returns null
//
// Begins playing 'filename' once, as a sound effect. If a file extension is not
// specified, it will be detected using the supported audio formats.
function PlaySound(filename : string) : PlaySound

/////////////////////
// WORLD FUNCTIONS //
/////////////////////

MAP_BACKGROUND   : 1                // background map layer
MAP_MIDDLEGROUND : 2                // middle map layer (still behind entities)
MAP_FOREGROUND   : 3                // foreground map layer (in front of entities)

// LoadWorld(filename : string) returns null
//
// Loads world 'filename' and begins playing. If a file extension is not
// specified, the standard world file extension will be used. This is a queued
// event; it will not occur immediately.
function LoadWorld(filename : string) : LoadWorld

// GetTile(layer : number, x : number, y : number) returns number
//
// Returns the id of the tile in map layer 'layer', located at 'x', 'y'. The
// coordinates are given in world units (e.g. pixels). The MAP_xxx constants
// can be used to specify the layer.
function GetTile(layer : number, x : number, y : number) : GetTile

// SetTile(layer : number, x : number, y : number, tileId : number) returns null
//
// Sets the id of the tile in map layer 'layer', located at 'x', 'y'. The
// coordinates are given in world units (e.g. pixels). The MAP_xxx constants
// can be used to specify the layer. Use tile id 0 to clear the map location.
function SetTile(layer : number, x : number, y : number, tileId : number) : SetTile

// IsFloor(x : number, y : number) returns boolean
//
// Returns true if the tile at 'x', 'y' in world coordinates is a sold tile, floor
// slope, or one-way floor.
function IsFloor(x : number, y : number) : IsFloor

//////////////////////
// ENTITY FUNCTIONS //
//////////////////////

// Spawn(template : string, properties : map) returns null
//
// Spawns a new entity, constructed with entity template 'template'.
// 'properties' contains entity instance-specific properties that are used to
// parameterize the entity template, such as entity location. This is a queued
// event; it will not occur immediately.
function Spawn(template : string, properties : map) : Spawn

// SpawnAt(template : string, properties : map) returns null
//
// Spawns a new entity, constructed with entity template 'template'. This is a
// convenience function for specifying the new entity's location at 'x', 'y'.
// It is equivalent to calling:
//   Spawn(template, rmerge({Location: {x: x, y: y}}, properties));
function SpawnAt(template : string, x : number, y : number, properties : map := {}) : SpawnAt

////////////////////////
// PARTICLE FUNCTIONS //
////////////////////////

// Spawns an immediate burst of 'count' particles at 'x', 'y' in world
// coordinates on layer 'layer', using the particle emitter description
// 'emitter'.
function ParticleBurst(emitter : map,
                       x       : number,
                       y       : number,
                       layer   : number,
                       count   : number) : ParticleBurst

// Spawns an ongoing "stream" of particles, emitted at 'rate' particles per
// second, at 'x', 'y' in world coordinates on layer 'layer', using the particle
// emitter description 'emitter'. The stream will last for 'lifetime'
// milliseconds before disappearing. If 'entityId' is set, the particle stream
// will follow the given entity's movements, maintaining its initial position
// offset.
function ParticleStream(emitter  : map,
                        x        : number,
                        y        : number,
                        layer    : number,
                        rate     : number,
                        lifetime : number,
                        entityId : id := null) : ParticleStream

////////////////////////////////////////////////////////////////////////////////
//                               KEEN GALAXY API                              //

// EnterLevel(name : string)
//
// Enters the new level 'name' from the map world. game.lastWin will be cleared.
function EnterLevel(name : string)
    StopMusic();
    PlaySound("enter_level");
    LoadWorld(name);
end function

// ExitLevel()
//
// Exits the current level for the map world, marking it as completed.
// game.lastWin will be set, and game.completed will be updated.
function ExitLevel()
    StopMusic();
    PlaySound("exit_level");
    LoadWorld("");
end function

// MessageBox(text : string, icon : string, align : string)
//
// Pauses the game and displays a message box on the screen. If an icon is
// displayed (e.g. a talking character) then 'align' specifies the side of the
// box where the text is displayed.
function MessageBox(text  : string,
                    icon  : string := "",
                    align : string := "right") : MessageBox

}